@@ -1,8 +1,10 @@
+from __future__ import annotations
+
 import dataclasses
 import enum
 import fractions
 import importlib.resources
-from typing import NamedTuple, Union
+from typing import ClassVar, Dict, NamedTuple, Union
 import warnings
 
 import numpy as np
@@ -161,7 +163,7 @@ def _load_materials_db(filename):
 # Shapes table
 #===============================================================================
 class ShapesTable():
-    def __init__(self, data: pd.DataFrame, units: pd.Series):
+    def __init__(self, data: pd.DataFrame, units: pd.Series, name: str = None):
         """
         Parameters
         ----------
@@ -169,9 +171,19 @@ class ShapesTable():
             Base data for the table.
         units : pd.Series
             Units for each (numeric) column in `data`.
+        name : str, optional
+            Name for this table.
         """
         self.data = data
         self.units = units
+        self.name = name
+
+    def __repr__(self):
+        clsname = f'{self.__class__.__module__}.{self.__class__.__name__}'
+        selfname = '' if self.name is None else f' {self.name!r}'
+        nshapes = len(self.data)
+
+        return f'<{clsname}{selfname} with {nshapes} shapes>'
 
     def get_prop(self, shape: str, prop: str):
         """Return a property from the table with units.
@@ -245,12 +257,14 @@ class ShapesTable():
         >>> lightest_shape(['W14X82', 'HSS4X4X1/2'])
         'HSS4X4X1/2'
         """
-        return self.data.loc[pd.Series(shape_list).str.casefold()].W.idxmin()
+        index = self.data.loc[pd.Series(shape_list).str.casefold()].W.idxmin()
+        return self.data.at[index, 'AISC_Manual_Label']
 
     @classmethod
     def from_file(cls,
                   file,
                   units,
+                  name=None,
                   true_values=TRUE_VALUES,
                   false_values=FALSE_VALUES,
                   na_values=NA_VALUES):
@@ -262,6 +276,8 @@ class ShapesTable():
             Name of the file to load.
         units : dict
             Dictionary of units, with keys corresponding to the column names.
+        name : str
+            Name to use for the created ShapesTable.
         true_values : list, optional
             List of values to convert to ``True``. (default: ['T'])
         false_values : list, optional
@@ -270,10 +286,10 @@ class ShapesTable():
             List of values to convert to ``nan``. (default: ['–']) (note that
             this is an en-dash U+2013, not an ASCII hyphen U+002D)
         """
-        data = pd.read_csv(file,
-                           true_values=true_values,
-                           false_values=false_values,
-                           na_values=na_values)
+        data: pd.DataFrame = pd.read_csv(file,
+                                         true_values=true_values,
+                                         false_values=false_values,
+                                         na_values=na_values)
         data.index = pd.Index(data['AISC_Manual_Label'].str.casefold(), name='')
 
         # Convert fractions to floats
@@ -290,185 +306,220 @@ class ShapesTable():
         data.sort_index(inplace=True)
         units = pd.Series(units)
         units.sort_index(inplace=True)
-        return cls(data, units)
+        return cls(data, units, name=name)
+
+    @staticmethod
+    def from_resource(key):
+        return _ShapesResource.from_registry(key).get_resource()
 
 
-_SHAPES_US_FILE = 'aisc-shapes-database-v15-0-US.csv.bz2'
-_SHAPES_US_UNITS = {
-    'W': 'lbf/ft',
-    'A': 'inch**2',
-    'd': 'inch',
-    'ddet': 'inch',
-    'Ht': 'inch',
-    'h': 'inch',
-    'OD': 'inch',
-    'bf': 'inch',
-    'bfdet': 'inch',
-    'B': 'inch',
-    'b': 'inch',
-    'ID': 'inch',
-    'tw': 'inch',
-    'twdet': 'inch',
-    'twdet/2': 'inch',
-    'tf': 'inch',
-    'tfdet': 'inch',
-    't': 'inch',
-    'tnom': 'inch',
-    'tdes': 'inch',
-    'kdes': 'inch',
-    'kdet': 'inch',
-    'k1': 'inch',
-    'x': 'inch',
-    'y': 'inch',
-    'eo': 'inch',
-    'xp': 'inch',
-    'yp': 'inch',
-    'bf/2tf': 'dimensionless',
-    'b/t': 'dimensionless',
-    'b/tdes': 'dimensionless',
-    'h/tw': 'dimensionless',
-    'h/tdes': 'dimensionless',
-    'D/t': 'dimensionless',
-    'Ix': 'inch**4',
-    'Zx': 'inch**3',
-    'Sx': 'inch**3',
-    'rx': 'inch',
-    'Iy': 'inch**4',
-    'Zy': 'inch**3',
-    'Sy': 'inch**3',
-    'ry': 'inch',
-    'Iz': 'inch**4',
-    'rz': 'inch',
-    'Sz': 'inch**3',
-    'J': 'inch**4',
-    'Cw': 'inch**6',
-    'C': 'inch**3',
-    'Wno': 'inch**2',
-    'Sw1': 'inch**4',
-    'Sw2': 'inch**4',
-    'Sw3': 'inch**4',
-    'Qf': 'inch**3',
-    'Qw': 'inch**3',
-    'ro': 'inch',
-    'H': 'dimensionless',
-    'tan(α)': 'dimensionless',
-    'Iw': 'inch**4',
-    'zA': 'inch',
-    'zB': 'inch',
-    'zC': 'inch',
-    'wA': 'inch',
-    'wB': 'inch',
-    'wC': 'inch',
-    'SzA': 'inch**3',
-    'SzB': 'inch**3',
-    'SzC': 'inch**3',
-    'SwA': 'inch**3',
-    'SwB': 'inch**3',
-    'SwC': 'inch**3',
-    'rts': 'inch',
-    'ho': 'inch',
-    'PA': 'inch',
-    'PA2': 'inch',
-    'PB': 'inch',
-    'PC': 'inch',
-    'PD': 'inch',
-    'T': 'inch',
-    'WGi': 'inch',
-    'WGo': 'inch',
-}
+@dataclasses.dataclass
+class _ShapesResource():
+    name: str
+    units: Dict[str, str]
+    filename: str
+    package: str = 'libtalley'
 
-_SHAPES_SI_FILE = 'aisc-shapes-database-v15-0-SI.csv.bz2'
-_SHAPES_SI_UNITS = {
-    'W': 'kg/m',
-    'A': 'mm**2',
-    'd': 'mm',
-    'ddet': 'mm',
-    'Ht': 'mm',
-    'h': 'mm',
-    'OD': 'mm',
-    'bf': 'mm',
-    'bfdet': 'mm',
-    'B': 'mm',
-    'b': 'mm',
-    'ID': 'mm',
-    'tw': 'mm',
-    'twdet': 'mm',
-    'twdet/2': 'mm',
-    'tf': 'mm',
-    'tfdet': 'mm',
-    't': 'mm',
-    'tnom': 'mm',
-    'tdes': 'mm',
-    'kdes': 'mm',
-    'kdet': 'mm',
-    'k1': 'mm',
-    'x': 'mm',
-    'y': 'mm',
-    'eo': 'mm',
-    'xp': 'mm',
-    'yp': 'mm',
-    'bf/2tf': 'dimensionless',
-    'b/t': 'dimensionless',
-    'b/tdes': 'dimensionless',
-    'h/tw': 'dimensionless',
-    'h/tdes': 'dimensionless',
-    'D/t': 'dimensionless',
-    'Ix': '1e6*mm**4',
-    'Zx': '1e3*mm**3',
-    'Sx': '1e3*mm**3',
-    'rx': 'mm',
-    'Iy': '1e6*mm**4',
-    'Zy': '1e3*mm**3',
-    'Sy': '1e3*mm**3',
-    'ry': 'mm',
-    'Iz': '1e6*mm**4',
-    'rz': 'mm',
-    'Sz': '1e3*mm**3',
-    'J': '1e3*mm**4',
-    'Cw': '1e9*mm**6',
-    'C': '1e3*mm**3',
-    'Wno': 'mm**2',
-    'Sw1': '1e6*mm**4',
-    'Sw2': '1e6*mm**4',
-    'Sw3': '1e6*mm**4',
-    'Qf': '1e3*mm**3',
-    'Qw': '1e3*mm**3',
-    'ro': 'mm',
-    'H': 'dimensionless',
-    'tan(α)': 'dimensionless',
-    'Iw': '1e6*mm**4',
-    'zA': 'mm',
-    'zB': 'mm',
-    'zC': 'mm',
-    'wA': 'mm',
-    'wB': 'mm',
-    'wC': 'mm',
-    'SzA': '1e3*mm**3',
-    'SzB': '1e3*mm**3',
-    'SzC': '1e3*mm**3',
-    'SwA': '1e3*mm**3',
-    'SwB': '1e3*mm**3',
-    'SwC': '1e3*mm**3',
-    'rts': 'mm',
-    'ho': 'mm',
-    'PA': 'mm',
-    'PA2': 'mm',
-    'PB': 'mm',
-    'PC': 'mm',
-    'PD': 'mm',
-    'T': 'mm',
-    'WGi': 'mm',
-    'WGo': 'mm',
-}
+    _registry: ClassVar[Dict[str, _ShapesResource]] = {}
+
+    def load_resource(self):
+        with importlib.resources.path(self.package, self.filename) as p:
+            return ShapesTable.from_file(p, self.units, name=self.name)
+
+    def get_resource(self):
+        try:
+            resource = self._resource
+        except AttributeError:
+            resource = self._resource = self.load_resource()
+
+        return resource
+
+    @classmethod
+    def register(cls, key, name, units, filename, package='libtalley'):
+        if key in cls._registry:
+            raise ValueError(f'resource with key {key!r} is already registered')
+
+        cls._registry[key] = cls(name, units, filename, package)
+
+    @classmethod
+    def from_registry(cls, key):
+        return cls._registry[key]
 
 
-def _load_shapes_db(filename, units):
-    with importlib.resources.path('libtalley', filename) as p:
-        return ShapesTable.from_file(p, units)
+_ShapesResource.register('US',
+                         name='AISC Shapes Database v15.0 (US)',
+                         filename='aisc-shapes-database-v15-0-US.csv.bz2',
+                         units={
+                             'W': 'lbf/ft',
+                             'A': 'inch**2',
+                             'd': 'inch',
+                             'ddet': 'inch',
+                             'Ht': 'inch',
+                             'h': 'inch',
+                             'OD': 'inch',
+                             'bf': 'inch',
+                             'bfdet': 'inch',
+                             'B': 'inch',
+                             'b': 'inch',
+                             'ID': 'inch',
+                             'tw': 'inch',
+                             'twdet': 'inch',
+                             'twdet/2': 'inch',
+                             'tf': 'inch',
+                             'tfdet': 'inch',
+                             't': 'inch',
+                             'tnom': 'inch',
+                             'tdes': 'inch',
+                             'kdes': 'inch',
+                             'kdet': 'inch',
+                             'k1': 'inch',
+                             'x': 'inch',
+                             'y': 'inch',
+                             'eo': 'inch',
+                             'xp': 'inch',
+                             'yp': 'inch',
+                             'bf/2tf': 'dimensionless',
+                             'b/t': 'dimensionless',
+                             'b/tdes': 'dimensionless',
+                             'h/tw': 'dimensionless',
+                             'h/tdes': 'dimensionless',
+                             'D/t': 'dimensionless',
+                             'Ix': 'inch**4',
+                             'Zx': 'inch**3',
+                             'Sx': 'inch**3',
+                             'rx': 'inch',
+                             'Iy': 'inch**4',
+                             'Zy': 'inch**3',
+                             'Sy': 'inch**3',
+                             'ry': 'inch',
+                             'Iz': 'inch**4',
+                             'rz': 'inch',
+                             'Sz': 'inch**3',
+                             'J': 'inch**4',
+                             'Cw': 'inch**6',
+                             'C': 'inch**3',
+                             'Wno': 'inch**2',
+                             'Sw1': 'inch**4',
+                             'Sw2': 'inch**4',
+                             'Sw3': 'inch**4',
+                             'Qf': 'inch**3',
+                             'Qw': 'inch**3',
+                             'ro': 'inch',
+                             'H': 'dimensionless',
+                             'tan(α)': 'dimensionless',
+                             'Iw': 'inch**4',
+                             'zA': 'inch',
+                             'zB': 'inch',
+                             'zC': 'inch',
+                             'wA': 'inch',
+                             'wB': 'inch',
+                             'wC': 'inch',
+                             'SzA': 'inch**3',
+                             'SzB': 'inch**3',
+                             'SzC': 'inch**3',
+                             'SwA': 'inch**3',
+                             'SwB': 'inch**3',
+                             'SwC': 'inch**3',
+                             'rts': 'inch',
+                             'ho': 'inch',
+                             'PA': 'inch',
+                             'PA2': 'inch',
+                             'PB': 'inch',
+                             'PC': 'inch',
+                             'PD': 'inch',
+                             'T': 'inch',
+                             'WGi': 'inch',
+                             'WGo': 'inch',
+                         })
 
+_ShapesResource.register('SI',
+                         name='AISC Shapes Database v15.0 (SI)',
+                         filename='aisc-shapes-database-v15-0-SI.csv.bz2',
+                         units={
+                             'W': 'kg/m',
+                             'A': 'mm**2',
+                             'd': 'mm',
+                             'ddet': 'mm',
+                             'Ht': 'mm',
+                             'h': 'mm',
+                             'OD': 'mm',
+                             'bf': 'mm',
+                             'bfdet': 'mm',
+                             'B': 'mm',
+                             'b': 'mm',
+                             'ID': 'mm',
+                             'tw': 'mm',
+                             'twdet': 'mm',
+                             'twdet/2': 'mm',
+                             'tf': 'mm',
+                             'tfdet': 'mm',
+                             't': 'mm',
+                             'tnom': 'mm',
+                             'tdes': 'mm',
+                             'kdes': 'mm',
+                             'kdet': 'mm',
+                             'k1': 'mm',
+                             'x': 'mm',
+                             'y': 'mm',
+                             'eo': 'mm',
+                             'xp': 'mm',
+                             'yp': 'mm',
+                             'bf/2tf': 'dimensionless',
+                             'b/t': 'dimensionless',
+                             'b/tdes': 'dimensionless',
+                             'h/tw': 'dimensionless',
+                             'h/tdes': 'dimensionless',
+                             'D/t': 'dimensionless',
+                             'Ix': '1e6*mm**4',
+                             'Zx': '1e3*mm**3',
+                             'Sx': '1e3*mm**3',
+                             'rx': 'mm',
+                             'Iy': '1e6*mm**4',
+                             'Zy': '1e3*mm**3',
+                             'Sy': '1e3*mm**3',
+                             'ry': 'mm',
+                             'Iz': '1e6*mm**4',
+                             'rz': 'mm',
+                             'Sz': '1e3*mm**3',
+                             'J': '1e3*mm**4',
+                             'Cw': '1e9*mm**6',
+                             'C': '1e3*mm**3',
+                             'Wno': 'mm**2',
+                             'Sw1': '1e6*mm**4',
+                             'Sw2': '1e6*mm**4',
+                             'Sw3': '1e6*mm**4',
+                             'Qf': '1e3*mm**3',
+                             'Qw': '1e3*mm**3',
+                             'ro': 'mm',
+                             'H': 'dimensionless',
+                             'tan(α)': 'dimensionless',
+                             'Iw': '1e6*mm**4',
+                             'zA': 'mm',
+                             'zB': 'mm',
+                             'zC': 'mm',
+                             'wA': 'mm',
+                             'wB': 'mm',
+                             'wC': 'mm',
+                             'SzA': '1e3*mm**3',
+                             'SzB': '1e3*mm**3',
+                             'SzC': '1e3*mm**3',
+                             'SwA': '1e3*mm**3',
+                             'SwB': '1e3*mm**3',
+                             'SwC': '1e3*mm**3',
+                             'rts': 'mm',
+                             'ho': 'mm',
+                             'PA': 'mm',
+                             'PA2': 'mm',
+                             'PB': 'mm',
+                             'PC': 'mm',
+                             'PD': 'mm',
+                             'T': 'mm',
+                             'WGi': 'mm',
+                             'WGo': 'mm',
+                         })
 
-shapes_US = _load_shapes_db(_SHAPES_US_FILE, _SHAPES_US_UNITS)
-shapes_SI = _load_shapes_db(_SHAPES_SI_FILE, _SHAPES_SI_UNITS)
+shapes_US = ShapesTable.from_resource('US')
+shapes_SI = ShapesTable.from_resource('SI')
 
 
 def property_lookup(shape, prop):
@@ -483,7 +534,9 @@ def property_lookup(shape, prop):
     prop : str
         Name of the property to look up.
     """
-    return shapes_US.data.loc[shape][prop]
+    warnings.warn('Replace `property_lookup(shape, prop)` with '
+                  '`shapes_US.get_prop(shape, prop)`')
+    return shapes_US.data.at[str(shape).casefold(), prop]
 
 
 def lightest_shape(shape_list):
@@ -505,6 +558,8 @@ def lightest_shape(shape_list):
     >>> lightest_shape(['W14X82', 'HSS4X4X1/2'])
     'HSS4X4X1/2'
     """
+    warnings.warn('Replace `lightest_shape(shape_list)` with '
+                  '`shapes_US.lightest_shape(shape_list)`')
     return shapes_US.lightest_shape(shape_list)
 
 
